package czerwony.kapturek.jbspell.ispell;


import java.io.*;
import java.util.*;
import javax.swing.*;

import czerwony.kapturek.jbspell.*;

import com.borland.primetime.ide.*;
import com.borland.primetime.properties.*;
import com.borland.primetime.util.*;


public class ISpellChecker extends AbstractChecker
{
    public static ISpellChecker I = null;

    Process ispellProcess = null;

    private BufferedReader in = null;
    private PrintWriter out;

    private String lastWord;
    private String[] lastSuggestions;

    private ISpellChecker()
    {
        printWelcomeMessage();
    }

    public static ISpellChecker getInstance()
    {
        if (I == null)
        {
            I = new ISpellChecker();
        }
        return I;
    }


    public static void initOpenTool( byte major, byte minor )
    {
        PropertyManager.registerPropertyGroup(new Settings());
        SpellCheckerTool.registerSpellCheckerImpl("ispell", "czerwony.kapturek.jbspell.ispell.ISpellChecker", Settings.TOPIC);
    }


    private void printWelcomeMessage()
    {
        System.out.println(
            "**********************************************************************\n" +
            "**  Initializing: jbSpell-ispell (v0.1)                             **\n" +
            "**                by Czerwony Kapturek (czerwony.kapturek@wp.pl)    **\n" +
            "**********************************************************************"
        );
    }


    public ConfigPanel getConfigPanel()
    {
        /**@todo: implement this czerwony.kapturek.jbspell.AbstractChecker abstract method*/
        return null;
    }

    public void addToDictionary(String word)
    {
        System.err.println("ISpellChecker.addToDictionary(" + word +")");
        ensureInitialized();
        out.println("*" + word);
        out.flush();
        out.println("#");
        out.flush();
    }

    public void ignore(String word)
    {
//        System.err.println("ISpellChecker.ignore()");
    }

    public boolean isInDictionary(String word)
    {
        ensureInitialized();
        try
        {
//            System.err.println("----------------------------");
//            System.err.println("isInDictionary(): " + word);
//            System.err.println("----------------------------");
            lastSuggestions = null;
            out.println(word);
            out.flush();
            String resp = in.readLine();
            String r2 = in.readLine();
            if ((resp == null) || (resp.length() == 0))
            {
                resp = r2;
            }
//            System.err.println("r2: "+r2);
//            System.err.println("resp: " + resp);
            if (resp == null || resp.length()==0)
            {
//                System.err.println("BLAD");
                return true;
            }
            char code = resp.charAt(0);
            if (code == '#')
            {
                lastSuggestions = new String[0];
            }
            else if ((code == '?') || (code == '&'))
            {
                StringTokenizer st = new StringTokenizer(resp.substring(resp.indexOf(':')+1), ",");
                lastSuggestions = new String[st.countTokens()];
                for (int i = 0; i < lastSuggestions.length; i++)
                {
                    lastSuggestions[i] = st.nextToken().trim();
                }
            }
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }

        return (lastSuggestions == null);
    }

    public String[] getSuggestions(String word)
    {
        ensureInitialized();
        if (!word.equals(lastWord))
        {
            isInDictionary(word);
        }
        return lastSuggestions;
    }


    private void ensureInitialized()
    {
        try
        {
            if (Settings.restartISpell || ispellProcess==null)
            {
                if (ispellProcess != null)
                {
                    ispellProcess.destroy();
                }

                String exe = Settings.executablePath.getValue();

                FastStringBuffer cmdLine = new FastStringBuffer(exe);
                cmdLine.append(' ');
                if (Settings.useAltDict.getBoolean())
                {
                    cmdLine.append("-d");
                    cmdLine.append(Settings.altDict.getValue().replace('\\', '/'));
                    cmdLine.append(' ');
                }
                if (Settings.useAltPersonalDict.getBoolean())
                {
                    cmdLine.append("-p");
                    cmdLine.append(Settings.altPersonalDict.getValue().replace('\\', '/'));
                    cmdLine.append(' ');
                }
                cmdLine.append("-a ");
                if (Settings.option_B.getBoolean())
                {
                    cmdLine.append("-B ");
                }
                if (Settings.option_C.getBoolean())
                {
                    cmdLine.append("-C ");
                }
                if (Settings.option_m.getBoolean())
                {
                    cmdLine.append("-m ");
                }
                if (Settings.option_P.getBoolean())
                {
                    cmdLine.append("-P ");
                }
                if (Settings.option_S.getBoolean())
                {
                    cmdLine.append("-S ");
                }

                String cmd = cmdLine.toString();
//                System.err.println(cmd);
                ispellProcess = Runtime.getRuntime().exec(cmd);
                in = new BufferedReader(new InputStreamReader(ispellProcess.getInputStream()));
                out = new PrintWriter(ispellProcess.getOutputStream());
                String greetings = in.readLine();
//                System.err.println(greetings);
                Settings.restartISpell = false;
            }
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            JOptionPane.showMessageDialog(Browser.getActiveBrowser(), "Cannot start ispell, please check configuration.");
            boolean b = PropertyManager.showPropertyDialog(Browser.getActiveBrowser(), "", czerwony.kapturek.jbspell.ispell.Settings.TOPIC, "");
        }
    }
}


